#### Code to fit an LDA model on the jbnst data (Lüdering & Winker, 2016)
# 
# 

library("tm")
library("topicmodels")
library("xlsx") # output the results to an xslx file

# Load the document by termmatrix
load(file="FinalTermmatrix.robj")


# constants for the estimation
k <- 165 # number of topics to be used in the 
BURNIN <- 1000 # Discards the first 1000 observations
THIN <- 100 # every 100th iterations returned for 
ITER <- 1000 # 1000 more iterations

# Fit and LDA model. Note: There is randomness involved - you may fix a random seed with "seed = 1"
#
# Note this will take a while
GibbsModel = LDA(termmatrix,k=k,method="Gibbs",control=list(burnin= BURNIN, thin = THIN, iter = ITER))

#
# OUtput the topic probabilities for the model. These probabilities are used in the second step.
#
write.xlsx(posterior(GibbsModel)$topic, "TopicProbabilities.xlsx")


######
# Merge with (manually corrected) metadata
load(file="metadata.robj")
TopicProbs <- as.data.frame(posterior(GibbsModel)$topic)
TopicProbs$itemid <- row.names(TopicProbs)
TopicProbs <- merge(TopicProbs, metadata, by = "itemid")
# Aggregate across years and output data
dataset <- aggregate(TopicProbs,list(TopicProbs$jahradjust),mean)
#
# Clean
dataset$itemid <- NULL
dataset$volumeadjust <- NULL
dataset$jahradjust <- NULL
colnames(dataset)[1] <- "year"
write.xlsx(dataset,"dataset.xlsx")